/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: George F. Riley<riley@ece.gatech.edu>
 * Modifications Lukasz Kiedrowski <kiedrowski@itti.com.pl>
 * Based on ANIMATION_INTERFACE__H  (AnimationInterface.h)
 */

// Interface between ns3 and the network animator

#ifndef DTN_TOOL__H
#define DTN_TOOL__H

#include <string>
#include <map>

#include "ns3/ptr.h"
#include "ns3/net-device.h"
#include "ns3/nstime.h"
#include "ns3/log.h"
#include "ns3/node-list.h"

#include "ns3/mobility-model.h"  //LKIE itti 300910
#include "ns3/node-container.h" //LKIE itti 061010



struct movement //LKIE itti 170111
{
  uint32_t n1;  // meeting node     
  uint32_t n2;  // meeting node
  uint32_t t1;  // meeting start time
  uint32_t t2;  // meeting end time
};
typedef std::vector<struct movement> MeetEvents;
typedef std::map<uint32_t, uint32_t> MapNam;

namespace ns3 {
class NetModel;

/**
 * \brief Interface to network animator
 *
 * Provides functions that facilitate communications with an
 * external or internal network animator.
 */
class DTNTool
{
public:
/**
 * @brief Construct the animator interface. No arguments needed.
 */
  DTNTool ();
/**
 * @brief Destructor for the animator interface.
 */
  ~DTNTool ();
/**
 * @brief Specify that animation commands are to be written
 * to the specified output file.
 *
 * This call is used to write the animation information to a text
 * file that can later be used as input to the network animator tool.
 *
 * @param fn The name of the output file.
 * @returns true if successful open.
 */
  bool SetOutputFile (const std::string& fn);

/**
 * @brief Specify that animation commands are to be written to
 * a socket.
 *
 * This call is used to set the ns3 process in server mode, waiting
 * for a TCP connection from the animator.  This call will not
 * return until the animator connects in, or if the bind to the
 * specified port fails.
 *
 * @param port Port number to bind to.
 * @returns true if connection created, false if bind failed.
 */
  bool SetServerPort (uint16_t port);

/**
 * @brief Writes the topology information and sets up the appropriate
 *  animation packet tx callback
 *
 * Writes the topology information to the appropriate output, depending
 * on prior calls to SetOutputFile, SetServerPort, or SetInternalAnimation.
 * Then creates the callbacks needed for the animator to start processing
 * packets.
 *
 */
  void StartAnimation ();
//n4c dedicated
  int  WriteNSimple (const char*, uint32_t); //LKIE itti 220910
  void SetRadius (double); //LKIE itti 300910
  MapNam StartAnimationWifi (uint32_t, uint32_t, uint32_t, uint32_t); //LKIE itti 300910
  MapNam MapNames (uint32_t, uint32_t, uint32_t); //LKIE itti 081010

  void CourseChangedVelocity (const NodeContainer* nc);//( Ptr<const NodeContainer> );  //LKIE itti 061010
/**
 * @brief Closes the interface to the animator.
 *
 */
  void StopAnimation ();
//n4c dedicated
  Vector3D* ReadPositions (const std::string fileName);
  MeetEvents ReadMovement (const std::string fileName);//LKIE itti 170111
  void InitializeInteractiveTerminal (const std::string user, const std::string path_interactive);
  void ChangeNodeAttr ();
  uint32_t FindKeyByVal(uint32_t);
  std::vector<std::string> PathSplit (std::string str);


private:
  void CourseChanged (std::string context, Ptr<const MobilityModel> model);
  // Packet tx animation callback
  void DevTxTrace (std::string context, Ptr<const Packet> p,
                 Ptr<NetDevice> tx, Ptr<NetDevice> rx,
                 Time txTime, Time rxTime);
  // Write specified amount of data to the specified handle
  int  WriteN (int, const char*, uint32_t);
private:
  int       m_fHandle;  // File handle for output (-1 if none)
  NetModel* m_model;    // If non nil, points to the internal network model
                        // for the interlan animator
//n4c dedicated
  uint16_t  m_port;  //LKIE itti 300910
  double    m_circle_radius; //LKIE itti 300910
  MapNam    m_map_names;       //LKIE itti 111010
  std::string m_interactive_file;
};
}
#endif

